@@ -200,12 +200,8 @@ int sig;
 	}
 #if 0
 	/* if we are doing an i_read, restart it */
-#ifdef HAVE_SIGLONGJMP
-      if (env_valid && (sig != 0)) siglongjmp(env,2);
-#else
-      if (env_valid && (sig != 0)) longjmp(env,2);
-#endif  /* HAVE_SIGLONGJMP */
-#endif /* 0 */
+	if (env_valid && (sig != 0)) longjmp(env,2);
+#endif
 }
 
 /*ARGSUSED*/
